use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'bin/moose-outdated',
    'lib/Class/MOP.pm',
    'lib/Class/MOP/Attribute.pm',
    'lib/Class/MOP/Class.pm',
    'lib/Class/MOP/Class/Immutable/Trait.pm',
    'lib/Class/MOP/Deprecated.pm',
    'lib/Class/MOP/Instance.pm',
    'lib/Class/MOP/Method.pm',
    'lib/Class/MOP/Method/Accessor.pm',
    'lib/Class/MOP/Method/Constructor.pm',
    'lib/Class/MOP/Method/Generated.pm',
    'lib/Class/MOP/Method/Inlined.pm',
    'lib/Class/MOP/Method/Meta.pm',
    'lib/Class/MOP/Method/Wrapped.pm',
    'lib/Class/MOP/MiniTrait.pm',
    'lib/Class/MOP/Mixin.pm',
    'lib/Class/MOP/Mixin/AttributeCore.pm',
    'lib/Class/MOP/Mixin/HasAttributes.pm',
    'lib/Class/MOP/Mixin/HasMethods.pm',
    'lib/Class/MOP/Mixin/HasOverloads.pm',
    'lib/Class/MOP/Module.pm',
    'lib/Class/MOP/Object.pm',
    'lib/Class/MOP/Overload.pm',
    'lib/Class/MOP/Package.pm',
    'lib/Moose.pm',
    'lib/Moose/Conflicts.pm',
    'lib/Moose/Cookbook.pod',
    'lib/Moose/Cookbook/Basics/BankAccount_MethodModifiersAndSubclassing.pod',
    'lib/Moose/Cookbook/Basics/BinaryTree_AttributeFeatures.pod',
    'lib/Moose/Cookbook/Basics/BinaryTree_BuilderAndLazyBuild.pod',
    'lib/Moose/Cookbook/Basics/Company_Subtypes.pod',
    'lib/Moose/Cookbook/Basics/DateTime_ExtendingNonMooseParent.pod',
    'lib/Moose/Cookbook/Basics/Document_AugmentAndInner.pod',
    'lib/Moose/Cookbook/Basics/Genome_OverloadingSubtypesAndCoercion.pod',
    'lib/Moose/Cookbook/Basics/HTTP_SubtypesAndCoercion.pod',
    'lib/Moose/Cookbook/Basics/Immutable.pod',
    'lib/Moose/Cookbook/Basics/Person_BUILDARGSAndBUILD.pod',
    'lib/Moose/Cookbook/Basics/Point_AttributesAndSubclassing.pod',
    'lib/Moose/Cookbook/Extending/Debugging_BaseClassRole.pod',
    'lib/Moose/Cookbook/Extending/ExtensionOverview.pod',
    'lib/Moose/Cookbook/Extending/Mooseish_MooseSugar.pod',
    'lib/Moose/Cookbook/Legacy/Debugging_BaseClassReplacement.pod',
    'lib/Moose/Cookbook/Legacy/Labeled_AttributeMetaclass.pod',
    'lib/Moose/Cookbook/Legacy/Table_ClassMetaclass.pod',
    'lib/Moose/Cookbook/Meta/GlobRef_InstanceMetaclass.pod',
    'lib/Moose/Cookbook/Meta/Labeled_AttributeTrait.pod',
    'lib/Moose/Cookbook/Meta/PrivateOrPublic_MethodMetaclass.pod',
    'lib/Moose/Cookbook/Meta/Table_MetaclassTrait.pod',
    'lib/Moose/Cookbook/Meta/WhyMeta.pod',
    'lib/Moose/Cookbook/Roles/ApplicationToInstance.pod',
    'lib/Moose/Cookbook/Roles/Comparable_CodeReuse.pod',
    'lib/Moose/Cookbook/Roles/Restartable_AdvancedComposition.pod',
    'lib/Moose/Cookbook/Snack/Keywords.pod',
    'lib/Moose/Cookbook/Snack/Types.pod',
    'lib/Moose/Cookbook/Style.pod',
    'lib/Moose/Deprecated.pm',
    'lib/Moose/Exception.pm',
    'lib/Moose/Exception/AccessorMustReadWrite.pm',
    'lib/Moose/Exception/AddParameterizableTypeTakesParameterizableType.pm',
    'lib/Moose/Exception/AddRoleTakesAMooseMetaRoleInstance.pm',
    'lib/Moose/Exception/AddRoleToARoleTakesAMooseMetaRole.pm',
    'lib/Moose/Exception/ApplyTakesABlessedInstance.pm',
    'lib/Moose/Exception/AttachToClassNeedsAClassMOPClassInstanceOrASubclass.pm',
    'lib/Moose/Exception/AttributeConflictInRoles.pm',
    'lib/Moose/Exception/AttributeConflictInSummation.pm',
    'lib/Moose/Exception/AttributeExtensionIsNotSupportedInRoles.pm',
    'lib/Moose/Exception/AttributeIsRequired.pm',
    'lib/Moose/Exception/AttributeMustBeAnClassMOPMixinAttributeCoreOrSubclass.pm',
    'lib/Moose/Exception/AttributeNamesDoNotMatch.pm',
    'lib/Moose/Exception/AttributeValueIsNotAnObject.pm',
    'lib/Moose/Exception/AttributeValueIsNotDefined.pm',
    'lib/Moose/Exception/AutoDeRefNeedsArrayRefOrHashRef.pm',
    'lib/Moose/Exception/BadOptionFormat.pm',
    'lib/Moose/Exception/BothBuilderAndDefaultAreNotAllowed.pm',
    'lib/Moose/Exception/BuilderDoesNotExist.pm',
    'lib/Moose/Exception/BuilderMethodNotSupportedForAttribute.pm',
    'lib/Moose/Exception/BuilderMethodNotSupportedForInlineAttribute.pm',
    'lib/Moose/Exception/BuilderMustBeAMethodName.pm',
    'lib/Moose/Exception/CallingMethodOnAnImmutableInstance.pm',
    'lib/Moose/Exception/CallingReadOnlyMethodOnAnImmutableInstance.pm',
    'lib/Moose/Exception/CanExtendOnlyClasses.pm',
    'lib/Moose/Exception/CanOnlyConsumeRole.pm',
    'lib/Moose/Exception/CanOnlyWrapBlessedCode.pm',
    'lib/Moose/Exception/CanReblessOnlyIntoASubclass.pm',
    'lib/Moose/Exception/CanReblessOnlyIntoASuperclass.pm',
    'lib/Moose/Exception/CannotAddAdditionalTypeCoercionsToUnion.pm',
    'lib/Moose/Exception/CannotAddAsAnAttributeToARole.pm',
    'lib/Moose/Exception/CannotApplyBaseClassRolesToRole.pm',
    'lib/Moose/Exception/CannotAssignValueToReadOnlyAccessor.pm',
    'lib/Moose/Exception/CannotAugmentIfLocalMethodPresent.pm',
    'lib/Moose/Exception/CannotAugmentNoSuperMethod.pm',
    'lib/Moose/Exception/CannotAutoDerefWithoutIsa.pm',
    'lib/Moose/Exception/CannotAutoDereferenceTypeConstraint.pm',
    'lib/Moose/Exception/CannotCalculateNativeType.pm',
    'lib/Moose/Exception/CannotCallAnAbstractBaseMethod.pm',
    'lib/Moose/Exception/CannotCallAnAbstractMethod.pm',
    'lib/Moose/Exception/CannotCoerceAWeakRef.pm',
    'lib/Moose/Exception/CannotCoerceAttributeWhichHasNoCoercion.pm',
    'lib/Moose/Exception/CannotCreateHigherOrderTypeWithoutATypeParameter.pm',
    'lib/Moose/Exception/CannotCreateMethodAliasLocalMethodIsPresent.pm',
    'lib/Moose/Exception/CannotCreateMethodAliasLocalMethodIsPresentInClass.pm',
    'lib/Moose/Exception/CannotDelegateLocalMethodIsPresent.pm',
    'lib/Moose/Exception/CannotDelegateWithoutIsa.pm',
    'lib/Moose/Exception/CannotFindDelegateMetaclass.pm',
    'lib/Moose/Exception/CannotFindType.pm',
    'lib/Moose/Exception/CannotFindTypeGivenToMatchOnType.pm',
    'lib/Moose/Exception/CannotFixMetaclassCompatibility.pm',
    'lib/Moose/Exception/CannotGenerateInlineConstraint.pm',
    'lib/Moose/Exception/CannotInitializeMooseMetaRoleComposite.pm',
    'lib/Moose/Exception/CannotInlineTypeConstraintCheck.pm',
    'lib/Moose/Exception/CannotLocatePackageInINC.pm',
    'lib/Moose/Exception/CannotMakeMetaclassCompatible.pm',
    'lib/Moose/Exception/CannotOverrideALocalMethod.pm',
    'lib/Moose/Exception/CannotOverrideBodyOfMetaMethods.pm',
    'lib/Moose/Exception/CannotOverrideLocalMethodIsPresent.pm',
    'lib/Moose/Exception/CannotOverrideNoSuperMethod.pm',
    'lib/Moose/Exception/CannotRegisterUnnamedTypeConstraint.pm',
    'lib/Moose/Exception/CannotUseLazyBuildAndDefaultSimultaneously.pm',
    'lib/Moose/Exception/CircularReferenceInAlso.pm',
    'lib/Moose/Exception/ClassDoesNotHaveInitMeta.pm',
    'lib/Moose/Exception/ClassDoesTheExcludedRole.pm',
    'lib/Moose/Exception/ClassNamesDoNotMatch.pm',
    'lib/Moose/Exception/CloneObjectExpectsAnInstanceOfMetaclass.pm',
    'lib/Moose/Exception/CodeBlockMustBeACodeRef.pm',
    'lib/Moose/Exception/CoercingWithoutCoercions.pm',
    'lib/Moose/Exception/CoercionAlreadyExists.pm',
    'lib/Moose/Exception/CoercionNeedsTypeConstraint.pm',
    'lib/Moose/Exception/ConflictDetectedInCheckRoleExclusions.pm',
    'lib/Moose/Exception/ConflictDetectedInCheckRoleExclusionsInToClass.pm',
    'lib/Moose/Exception/ConstructClassInstanceTakesPackageName.pm',
    'lib/Moose/Exception/CouldNotCreateMethod.pm',
    'lib/Moose/Exception/CouldNotCreateWriter.pm',
    'lib/Moose/Exception/CouldNotEvalConstructor.pm',
    'lib/Moose/Exception/CouldNotEvalDestructor.pm',
    'lib/Moose/Exception/CouldNotFindTypeConstraintToCoerceFrom.pm',
    'lib/Moose/Exception/CouldNotGenerateInlineAttributeMethod.pm',
    'lib/Moose/Exception/CouldNotLocateTypeConstraintForUnion.pm',
    'lib/Moose/Exception/CouldNotParseType.pm',
    'lib/Moose/Exception/CreateMOPClassTakesArrayRefOfAttributes.pm',
    'lib/Moose/Exception/CreateMOPClassTakesArrayRefOfSuperclasses.pm',
    'lib/Moose/Exception/CreateMOPClassTakesHashRefOfMethods.pm',
    'lib/Moose/Exception/CreateTakesArrayRefOfRoles.pm',
    'lib/Moose/Exception/CreateTakesHashRefOfAttributes.pm',
    'lib/Moose/Exception/CreateTakesHashRefOfMethods.pm',
    'lib/Moose/Exception/DefaultToMatchOnTypeMustBeCodeRef.pm',
    'lib/Moose/Exception/DelegationToAClassWhichIsNotLoaded.pm',
    'lib/Moose/Exception/DelegationToARoleWhichIsNotLoaded.pm',
    'lib/Moose/Exception/DelegationToATypeWhichIsNotAClass.pm',
    'lib/Moose/Exception/DoesRequiresRoleName.pm',
    'lib/Moose/Exception/EnumCalledWithAnArrayRefAndAdditionalArgs.pm',
    'lib/Moose/Exception/EnumValuesMustBeString.pm',
    'lib/Moose/Exception/ExtendsMissingArgs.pm',
    'lib/Moose/Exception/HandlesMustBeAHashRef.pm',
    'lib/Moose/Exception/IllegalInheritedOptions.pm',
    'lib/Moose/Exception/IllegalMethodTypeToAddMethodModifier.pm',
    'lib/Moose/Exception/IncompatibleMetaclassOfSuperclass.pm',
    'lib/Moose/Exception/InitMetaRequiresClass.pm',
    'lib/Moose/Exception/InitializeTakesUnBlessedPackageName.pm',
    'lib/Moose/Exception/InstanceBlessedIntoWrongClass.pm',
    'lib/Moose/Exception/InstanceMustBeABlessedReference.pm',
    'lib/Moose/Exception/InvalidArgPassedToMooseUtilMetaRole.pm',
    'lib/Moose/Exception/InvalidArgumentToMethod.pm',
    'lib/Moose/Exception/InvalidArgumentsToTraitAliases.pm',
    'lib/Moose/Exception/InvalidBaseTypeGivenToCreateParameterizedTypeConstraint.pm',
    'lib/Moose/Exception/InvalidHandleValue.pm',
    'lib/Moose/Exception/InvalidHasProvidedInARole.pm',
    'lib/Moose/Exception/InvalidNameForType.pm',
    'lib/Moose/Exception/InvalidOverloadOperator.pm',
    'lib/Moose/Exception/InvalidRoleApplication.pm',
    'lib/Moose/Exception/InvalidTypeConstraint.pm',
    'lib/Moose/Exception/InvalidTypeGivenToCreateParameterizedTypeConstraint.pm',
    'lib/Moose/Exception/InvalidValueForIs.pm',
    'lib/Moose/Exception/IsaDoesNotDoTheRole.pm',
    'lib/Moose/Exception/IsaLacksDoesMethod.pm',
    'lib/Moose/Exception/LazyAttributeNeedsADefault.pm',
    'lib/Moose/Exception/Legacy.pm',
    'lib/Moose/Exception/MOPAttributeNewNeedsAttributeName.pm',
    'lib/Moose/Exception/MatchActionMustBeACodeRef.pm',
    'lib/Moose/Exception/MessageParameterMustBeCodeRef.pm',
    'lib/Moose/Exception/MetaclassIsAClassNotASubclassOfGivenMetaclass.pm',
    'lib/Moose/Exception/MetaclassIsARoleNotASubclassOfGivenMetaclass.pm',
    'lib/Moose/Exception/MetaclassIsNotASubclassOfGivenMetaclass.pm',
    'lib/Moose/Exception/MetaclassMustBeASubclassOfMooseMetaClass.pm',
    'lib/Moose/Exception/MetaclassMustBeASubclassOfMooseMetaRole.pm',
    'lib/Moose/Exception/MetaclassMustBeDerivedFromClassMOPClass.pm',
    'lib/Moose/Exception/MetaclassNotLoaded.pm',
    'lib/Moose/Exception/MetaclassTypeIncompatible.pm',
    'lib/Moose/Exception/MethodExpectedAMetaclassObject.pm',
    'lib/Moose/Exception/MethodExpectsFewerArgs.pm',
    'lib/Moose/Exception/MethodExpectsMoreArgs.pm',
    'lib/Moose/Exception/MethodModifierNeedsMethodName.pm',
    'lib/Moose/Exception/MethodNameConflictInRoles.pm',
    'lib/Moose/Exception/MethodNameNotFoundInInheritanceHierarchy.pm',
    'lib/Moose/Exception/MethodNameNotGiven.pm',
    'lib/Moose/Exception/MustDefineAMethodName.pm',
    'lib/Moose/Exception/MustDefineAnAttributeName.pm',
    'lib/Moose/Exception/MustDefineAnOverloadOperator.pm',
    'lib/Moose/Exception/MustHaveAtLeastOneValueToEnumerate.pm',
    'lib/Moose/Exception/MustPassAHashOfOptions.pm',
    'lib/Moose/Exception/MustPassAMooseMetaRoleInstanceOrSubclass.pm',
    'lib/Moose/Exception/MustPassAPackageNameOrAnExistingClassMOPPackageInstance.pm',
    'lib/Moose/Exception/MustPassEvenNumberOfArguments.pm',
    'lib/Moose/Exception/MustPassEvenNumberOfAttributeOptions.pm',
    'lib/Moose/Exception/MustProvideANameForTheAttribute.pm',
    'lib/Moose/Exception/MustSpecifyAtleastOneMethod.pm',
    'lib/Moose/Exception/MustSpecifyAtleastOneRole.pm',
    'lib/Moose/Exception/MustSpecifyAtleastOneRoleToApplicant.pm',
    'lib/Moose/Exception/MustSupplyAClassMOPAttributeInstance.pm',
    'lib/Moose/Exception/MustSupplyADelegateToMethod.pm',
    'lib/Moose/Exception/MustSupplyAMetaclass.pm',
    'lib/Moose/Exception/MustSupplyAMooseMetaAttributeInstance.pm',
    'lib/Moose/Exception/MustSupplyAnAccessorTypeToConstructWith.pm',
    'lib/Moose/Exception/MustSupplyAnAttributeToConstructWith.pm',
    'lib/Moose/Exception/MustSupplyArrayRefAsCurriedArguments.pm',
    'lib/Moose/Exception/MustSupplyPackageNameAndName.pm',
    'lib/Moose/Exception/NeedsTypeConstraintUnionForTypeCoercionUnion.pm',
    'lib/Moose/Exception/NeitherAttributeNorAttributeNameIsGiven.pm',
    'lib/Moose/Exception/NeitherClassNorClassNameIsGiven.pm',
    'lib/Moose/Exception/NeitherRoleNorRoleNameIsGiven.pm',
    'lib/Moose/Exception/NeitherTypeNorTypeNameIsGiven.pm',
    'lib/Moose/Exception/NoAttributeFoundInSuperClass.pm',
    'lib/Moose/Exception/NoBodyToInitializeInAnAbstractBaseClass.pm',
    'lib/Moose/Exception/NoCasesMatched.pm',
    'lib/Moose/Exception/NoConstraintCheckForTypeConstraint.pm',
    'lib/Moose/Exception/NoDestructorClassSpecified.pm',
    'lib/Moose/Exception/NoImmutableTraitSpecifiedForClass.pm',
    'lib/Moose/Exception/NoParentGivenToSubtype.pm',
    'lib/Moose/Exception/OnlyInstancesCanBeCloned.pm',
    'lib/Moose/Exception/OperatorIsRequired.pm',
    'lib/Moose/Exception/OverloadConflictInSummation.pm',
    'lib/Moose/Exception/OverloadRequiresAMetaClass.pm',
    'lib/Moose/Exception/OverloadRequiresAMetaMethod.pm',
    'lib/Moose/Exception/OverloadRequiresAMetaOverload.pm',
    'lib/Moose/Exception/OverloadRequiresAMethodNameOrCoderef.pm',
    'lib/Moose/Exception/OverloadRequiresAnOperator.pm',
    'lib/Moose/Exception/OverloadRequiresNamesForCoderef.pm',
    'lib/Moose/Exception/OverrideConflictInComposition.pm',
    'lib/Moose/Exception/OverrideConflictInSummation.pm',
    'lib/Moose/Exception/PackageDoesNotUseMooseExporter.pm',
    'lib/Moose/Exception/PackageNameAndNameParamsNotGivenToWrap.pm',
    'lib/Moose/Exception/PackagesAndModulesAreNotCachable.pm',
    'lib/Moose/Exception/ParameterIsNotSubtypeOfParent.pm',
    'lib/Moose/Exception/ReferencesAreNotAllowedAsDefault.pm',
    'lib/Moose/Exception/RequiredAttributeLacksInitialization.pm',
    'lib/Moose/Exception/RequiredAttributeNeedsADefault.pm',
    'lib/Moose/Exception/RequiredMethodsImportedByClass.pm',
    'lib/Moose/Exception/RequiredMethodsNotImplementedByClass.pm',
    'lib/Moose/Exception/Role/Attribute.pm',
    'lib/Moose/Exception/Role/AttributeName.pm',
    'lib/Moose/Exception/Role/Class.pm',
    'lib/Moose/Exception/Role/EitherAttributeOrAttributeName.pm',
    'lib/Moose/Exception/Role/Instance.pm',
    'lib/Moose/Exception/Role/InstanceClass.pm',
    'lib/Moose/Exception/Role/InvalidAttributeOptions.pm',
    'lib/Moose/Exception/Role/Method.pm',
    'lib/Moose/Exception/Role/ParamsHash.pm',
    'lib/Moose/Exception/Role/Role.pm',
    'lib/Moose/Exception/Role/RoleForCreate.pm',
    'lib/Moose/Exception/Role/RoleForCreateMOPClass.pm',
    'lib/Moose/Exception/Role/TypeConstraint.pm',
    'lib/Moose/Exception/RoleDoesTheExcludedRole.pm',
    'lib/Moose/Exception/RoleExclusionConflict.pm',
    'lib/Moose/Exception/RoleNameRequired.pm',
    'lib/Moose/Exception/RoleNameRequiredForMooseMetaRole.pm',
    'lib/Moose/Exception/RolesDoNotSupportAugment.pm',
    'lib/Moose/Exception/RolesDoNotSupportExtends.pm',
    'lib/Moose/Exception/RolesDoNotSupportInner.pm',
    'lib/Moose/Exception/RolesDoNotSupportRegexReferencesForMethodModifiers.pm',
    'lib/Moose/Exception/RolesInCreateTakesAnArrayRef.pm',
    'lib/Moose/Exception/RolesListMustBeInstancesOfMooseMetaRole.pm',
    'lib/Moose/Exception/SingleParamsToNewMustBeHashRef.pm',
    'lib/Moose/Exception/TriggerMustBeACodeRef.pm',
    'lib/Moose/Exception/TypeConstraintCannotBeUsedForAParameterizableType.pm',
    'lib/Moose/Exception/TypeConstraintIsAlreadyCreated.pm',
    'lib/Moose/Exception/TypeParameterMustBeMooseMetaType.pm',
    'lib/Moose/Exception/UnableToCanonicalizeHandles.pm',
    'lib/Moose/Exception/UnableToCanonicalizeNonRolePackage.pm',
    'lib/Moose/Exception/UnableToRecognizeDelegateMetaclass.pm',
    'lib/Moose/Exception/UndefinedHashKeysPassedToMethod.pm',
    'lib/Moose/Exception/UnionCalledWithAnArrayRefAndAdditionalArgs.pm',
    'lib/Moose/Exception/UnionTakesAtleastTwoTypeNames.pm',
    'lib/Moose/Exception/ValidationFailedForInlineTypeConstraint.pm',
    'lib/Moose/Exception/ValidationFailedForTypeConstraint.pm',
    'lib/Moose/Exception/WrapTakesACodeRefToBless.pm',
    'lib/Moose/Exception/WrongTypeConstraintGiven.pm',
    'lib/Moose/Exporter.pm',
    'lib/Moose/Intro.pod',
    'lib/Moose/Manual.pod',
    'lib/Moose/Manual/Attributes.pod',
    'lib/Moose/Manual/BestPractices.pod',
    'lib/Moose/Manual/Classes.pod',
    'lib/Moose/Manual/Concepts.pod',
    'lib/Moose/Manual/Construction.pod',
    'lib/Moose/Manual/Contributing.pod',
    'lib/Moose/Manual/Delegation.pod',
    'lib/Moose/Manual/Delta.pod',
    'lib/Moose/Manual/Exceptions.pod',
    'lib/Moose/Manual/Exceptions/Manifest.pod',
    'lib/Moose/Manual/FAQ.pod',
    'lib/Moose/Manual/MOP.pod',
    'lib/Moose/Manual/MethodModifiers.pod',
    'lib/Moose/Manual/MooseX.pod',
    'lib/Moose/Manual/Resources.pod',
    'lib/Moose/Manual/Roles.pod',
    'lib/Moose/Manual/Support.pod',
    'lib/Moose/Manual/Types.pod',
    'lib/Moose/Manual/Unsweetened.pod',
    'lib/Moose/Meta/Attribute.pm',
    'lib/Moose/Meta/Attribute/Native.pm',
    'lib/Moose/Meta/Attribute/Native/Trait.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Array.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Bool.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Code.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Counter.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Hash.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/Number.pm',
    'lib/Moose/Meta/Attribute/Native/Trait/String.pm',
    'lib/Moose/Meta/Class.pm',
    'lib/Moose/Meta/Class/Immutable/Trait.pm',
    'lib/Moose/Meta/Instance.pm',
    'lib/Moose/Meta/Method.pm',
    'lib/Moose/Meta/Method/Accessor.pm',
    'lib/Moose/Meta/Method/Accessor/Native.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/Writer.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/accessor.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/clear.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/count.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/delete.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/elements.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/first.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/first_index.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/get.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/grep.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/insert.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/is_empty.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/join.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/map.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/natatime.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/pop.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/push.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/reduce.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/set.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/shallow_clone.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/shift.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/shuffle.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/sort.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/sort_in_place.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/splice.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/uniq.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Array/unshift.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Bool/not.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Bool/set.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Bool/toggle.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Bool/unset.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Code/execute.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Code/execute_method.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Collection.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Counter/Writer.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Counter/dec.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Counter/inc.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Counter/reset.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Counter/set.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/Writer.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/accessor.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/clear.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/count.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/defined.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/delete.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/elements.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/exists.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/get.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/is_empty.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/keys.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/kv.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/set.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/shallow_clone.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Hash/values.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/abs.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/add.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/div.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/mod.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/mul.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/set.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Number/sub.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Reader.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/append.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/chomp.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/chop.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/clear.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/inc.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/length.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/match.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/prepend.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/replace.pm',
    'lib/Moose/Meta/Method/Accessor/Native/String/substr.pm',
    'lib/Moose/Meta/Method/Accessor/Native/Writer.pm',
    'lib/Moose/Meta/Method/Augmented.pm',
    'lib/Moose/Meta/Method/Constructor.pm',
    'lib/Moose/Meta/Method/Delegation.pm',
    'lib/Moose/Meta/Method/Destructor.pm',
    'lib/Moose/Meta/Method/Meta.pm',
    'lib/Moose/Meta/Method/Overridden.pm',
    'lib/Moose/Meta/Mixin/AttributeCore.pm',
    'lib/Moose/Meta/Object/Trait.pm',
    'lib/Moose/Meta/Role.pm',
    'lib/Moose/Meta/Role/Application.pm',
    'lib/Moose/Meta/Role/Application/RoleSummation.pm',
    'lib/Moose/Meta/Role/Application/ToClass.pm',
    'lib/Moose/Meta/Role/Application/ToInstance.pm',
    'lib/Moose/Meta/Role/Application/ToRole.pm',
    'lib/Moose/Meta/Role/Attribute.pm',
    'lib/Moose/Meta/Role/Composite.pm',
    'lib/Moose/Meta/Role/Method.pm',
    'lib/Moose/Meta/Role/Method/Conflicting.pm',
    'lib/Moose/Meta/Role/Method/Required.pm',
    'lib/Moose/Meta/TypeCoercion.pm',
    'lib/Moose/Meta/TypeCoercion/Union.pm',
    'lib/Moose/Meta/TypeConstraint.pm',
    'lib/Moose/Meta/TypeConstraint/Class.pm',
    'lib/Moose/Meta/TypeConstraint/DuckType.pm',
    'lib/Moose/Meta/TypeConstraint/Enum.pm',
    'lib/Moose/Meta/TypeConstraint/Parameterizable.pm',
    'lib/Moose/Meta/TypeConstraint/Parameterized.pm',
    'lib/Moose/Meta/TypeConstraint/Registry.pm',
    'lib/Moose/Meta/TypeConstraint/Role.pm',
    'lib/Moose/Meta/TypeConstraint/Union.pm',
    'lib/Moose/Object.pm',
    'lib/Moose/Role.pm',
    'lib/Moose/Spec/Role.pod',
    'lib/Moose/Unsweetened.pod',
    'lib/Moose/Util.pm',
    'lib/Moose/Util/MetaRole.pm',
    'lib/Moose/Util/TypeConstraints.pm',
    'lib/Moose/Util/TypeConstraints/Builtins.pm',
    'lib/Test/Moose.pm',
    'lib/metaclass.pm',
    'lib/oose.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/000_load.t',
    't/attributes/accessor_context.t',
    't/attributes/accessor_inlining.t',
    't/attributes/accessor_override_method.t',
    't/attributes/accessor_overwrite_warning.t',
    't/attributes/attr_dereference_test.t',
    't/attributes/attribute_accessor_generation.t',
    't/attributes/attribute_custom_metaclass.t',
    't/attributes/attribute_delegation.t',
    't/attributes/attribute_does.t',
    't/attributes/attribute_inherited_slot_specs.t',
    't/attributes/attribute_lazy_initializer.t',
    't/attributes/attribute_names.t',
    't/attributes/attribute_reader_generation.t',
    't/attributes/attribute_required.t',
    't/attributes/attribute_traits.t',
    't/attributes/attribute_traits_n_meta.t',
    't/attributes/attribute_traits_parameterized.t',
    't/attributes/attribute_traits_registered.t',
    't/attributes/attribute_triggers.t',
    't/attributes/attribute_type_unions.t',
    't/attributes/attribute_without_any_methods.t',
    't/attributes/attribute_writer_generation.t',
    't/attributes/bad_coerce.t',
    't/attributes/chained_coercion.t',
    't/attributes/clone_weak.t',
    't/attributes/default_class_role_types.t',
    't/attributes/default_undef.t',
    't/attributes/delegation_and_modifiers.t',
    't/attributes/delegation_arg_aliasing.t',
    't/attributes/delegation_target_not_loaded.t',
    't/attributes/illegal_options_for_inheritance.t',
    't/attributes/inherit_lazy_build.t',
    't/attributes/lazy_no_default.t',
    't/attributes/method_generation_rules.t',
    't/attributes/misc_attribute_coerce_lazy.t',
    't/attributes/misc_attribute_tests.t',
    't/attributes/more_attr_delegation.t',
    't/attributes/no_init_arg.t',
    't/attributes/no_slot_access.t',
    't/attributes/non_alpha_attr_names.t',
    't/attributes/numeric_defaults.t',
    't/attributes/trigger_and_coerce.t',
    't/attributes/type_constraint.t',
    't/basics/always_strict_warnings.t',
    't/basics/basic_class_setup.t',
    't/basics/buildargs.t',
    't/basics/buildargs_warning.t',
    't/basics/create.t',
    't/basics/create_anon.t',
    't/basics/definition_context.t',
    't/basics/deprecations.t',
    't/basics/destruction.t',
    't/basics/error_handling.t',
    't/basics/global-destruction-helper.pl',
    't/basics/global_destruction.t',
    't/basics/import_unimport.t',
    't/basics/inner_and_augment.t',
    't/basics/load_into_main.t',
    't/basics/method_modifier_with_regexp.t',
    't/basics/methods.t',
    't/basics/moose_object_does.t',
    't/basics/moose_respects_type_constraints.t',
    't/basics/override_and_foreign_classes.t',
    't/basics/override_augment_inner_super.t',
    't/basics/rebless.t',
    't/basics/require_superclasses.t',
    't/basics/super_and_override.t',
    't/basics/super_warns_on_args.t',
    't/basics/universal_methods_wrappable.t',
    't/basics/wrapped_method_cxt_propagation.t',
    't/bugs/DEMOLISHALL.t',
    't/bugs/DEMOLISHALL_shortcutted.t',
    't/bugs/DEMOLISH_eats_exceptions.t',
    't/bugs/DEMOLISH_eats_mini.t',
    't/bugs/DEMOLISH_fails_without_metaclass.t',
    't/bugs/Moose_Object_error.t',
    't/bugs/anon_method_metaclass.t',
    't/bugs/application_metarole_compat.t',
    't/bugs/apply_role_to_one_instance_only.t',
    't/bugs/attribute_trait_parameters.t',
    't/bugs/augment_recursion_bug.t',
    't/bugs/coerce_without_coercion.t',
    't/bugs/constructor_object_overload.t',
    't/bugs/create_anon_recursion.t',
    't/bugs/create_anon_role_pass.t',
    't/bugs/delete_sub_stash.t',
    't/bugs/find_custom_trait_rt_92089.t',
    't/bugs/handles_foreign_class_bug.t',
    't/bugs/immutable_metaclass_does_role.t',
    't/bugs/immutable_n_default_x2.t',
    't/bugs/inheriting_from_roles.t',
    't/bugs/inline_reader_bug.t',
    't/bugs/instance_application_role_args.t',
    't/bugs/lazybuild_required_undef.t',
    't/bugs/mark_as_methods_overloading_breakage.t',
    't/bugs/moo_delegation.t',
    't/bugs/moose_exporter_false_circular_reference_rt_63818.t',
    't/bugs/moose_octal_defaults.t',
    't/bugs/native_trait_handles_bad_value.t',
    't/bugs/overloading_edge_cases.t',
    't/bugs/reader_precedence_bug.t',
    't/bugs/role_caller.t',
    't/bugs/subclass_use_base_bug.t',
    't/bugs/subtype_conflict_bug.t',
    't/bugs/subtype_quote_bug.t',
    't/bugs/super_recursion.t',
    't/bugs/traits_with_exporter.t',
    't/bugs/type_constraint_messages.t',
    't/cmop/ArrayBasedStorage_test.t',
    't/cmop/AttributesWithHistory_test.t',
    't/cmop/BinaryTree_test.t',
    't/cmop/C3MethodDispatchOrder_test.t',
    't/cmop/ClassEncapsulatedAttributes_test.t',
    't/cmop/Class_C3_compatibility.t',
    't/cmop/InsideOutClass_test.t',
    't/cmop/InstanceCountingClass_test.t',
    't/cmop/LazyClass_test.t',
    't/cmop/Perl6Attribute_test.t',
    't/cmop/RT_27329_fix.t',
    't/cmop/RT_39001_fix.t',
    't/cmop/RT_41255.t',
    't/cmop/add_attribute_alternate.t',
    't/cmop/add_method_debugmode.t',
    't/cmop/add_method_modifier.t',
    't/cmop/advanced_methods.t',
    't/cmop/anon_class.t',
    't/cmop/anon_class_create_init.t',
    't/cmop/anon_class_keep_alive.t',
    't/cmop/anon_class_leak.t',
    't/cmop/anon_class_removal.t',
    't/cmop/anon_packages.t',
    't/cmop/attribute.t',
    't/cmop/attribute_duplication.t',
    't/cmop/attribute_errors_and_edge_cases.t',
    't/cmop/attribute_get_read_write.t',
    't/cmop/attribute_initializer.t',
    't/cmop/attribute_introspection.t',
    't/cmop/attribute_non_alpha_name.t',
    't/cmop/attributes.t',
    't/cmop/basic.t',
    't/cmop/before_after_dollar_under.t',
    't/cmop/class_errors_and_edge_cases.t',
    't/cmop/class_is_pristine.t',
    't/cmop/class_precedence_list.t',
    't/cmop/constant_codeinfo.t',
    't/cmop/create_class.t',
    't/cmop/custom_instance.t',
    't/cmop/deprecated.t',
    't/cmop/get_code_info.t',
    't/cmop/immutable_custom_trait.t',
    't/cmop/immutable_metaclass.t',
    't/cmop/immutable_w_constructors.t',
    't/cmop/immutable_w_custom_metaclass.t',
    't/cmop/inline_and_dollar_at.t',
    't/cmop/inline_structor.t',
    't/cmop/insertion_order.t',
    't/cmop/instance.t',
    't/cmop/instance_inline.t',
    't/cmop/instance_metaclass_incompat.t',
    't/cmop/instance_metaclass_incompat_dyn.t',
    't/cmop/lib/ArrayBasedStorage.pm',
    't/cmop/lib/AttributesWithHistory.pm',
    't/cmop/lib/BinaryTree.pm',
    't/cmop/lib/C3MethodDispatchOrder.pm',
    't/cmop/lib/ClassEncapsulatedAttributes.pm',
    't/cmop/lib/InsideOutClass.pm',
    't/cmop/lib/InstanceCountingClass.pm',
    't/cmop/lib/LazyClass.pm',
    't/cmop/lib/MyMetaClass.pm',
    't/cmop/lib/MyMetaClass/Attribute.pm',
    't/cmop/lib/MyMetaClass/Instance.pm',
    't/cmop/lib/MyMetaClass/Method.pm',
    't/cmop/lib/MyMetaClass/Random.pm',
    't/cmop/lib/Perl6Attribute.pm',
    't/cmop/lib/SyntaxError.pm',
    't/cmop/load.t',
    't/cmop/magic.t',
    't/cmop/make_mutable.t',
    't/cmop/meta_method.t',
    't/cmop/meta_package.t',
    't/cmop/meta_package_extension.t',
    't/cmop/metaclass.t',
    't/cmop/metaclass_incompatibility.t',
    't/cmop/metaclass_incompatibility_dyn.t',
    't/cmop/metaclass_inheritance.t',
    't/cmop/metaclass_loads_classes.t',
    't/cmop/metaclass_reinitialize.t',
    't/cmop/method.t',
    't/cmop/method_modifiers.t',
    't/cmop/methods.t',
    't/cmop/modify_parent_method.t',
    't/cmop/new_and_clone_metaclasses.t',
    't/cmop/null_stash.t',
    't/cmop/numeric_defaults.t',
    't/cmop/package_variables.t',
    't/cmop/random_eval_bug.t',
    't/cmop/rebless_instance.t',
    't/cmop/rebless_instance_away.t',
    't/cmop/rebless_overload.t',
    't/cmop/rebless_with_extra_params.t',
    't/cmop/scala_style_mixin_composition.t',
    't/cmop/self_introspection.t',
    't/cmop/subclasses.t',
    't/cmop/subname.t',
    't/cmop/universal_methods.t',
    't/compat/composite_metaroles.t',
    't/compat/extends_nonmoose_that_isa_moose_with_metarole.t',
    't/compat/foreign_inheritence.t',
    't/compat/inc_hash.t',
    't/compat/module_refresh_compat.t',
    't/compat/moose_respects_base.t',
    't/examples/Child_Parent_attr_inherit.t',
    't/examples/example1.t',
    't/examples/example2.t',
    't/examples/example_Moose_POOP.t',
    't/examples/example_Protomoose.t',
    't/examples/example_w_DCS.t',
    't/examples/example_w_TestDeep.t',
    't/examples/record_set_iterator.t',
    't/exceptions/attribute.t',
    't/exceptions/class-mop-attribute.t',
    't/exceptions/class-mop-class-immutable-trait.t',
    't/exceptions/class-mop-class.t',
    't/exceptions/class-mop-method-accessor.t',
    't/exceptions/class-mop-method-constructor.t',
    't/exceptions/class-mop-method-generated.t',
    't/exceptions/class-mop-method-meta.t',
    't/exceptions/class-mop-method-wrapped.t',
    't/exceptions/class-mop-method.t',
    't/exceptions/class-mop-mixin-hasattributes.t',
    't/exceptions/class-mop-mixin-hasmethods.t',
    't/exceptions/class-mop-module.t',
    't/exceptions/class-mop-object.t',
    't/exceptions/class-mop-package.t',
    't/exceptions/class.t',
    't/exceptions/cmop.t',
    't/exceptions/exception-lazyattributeneedsadefault.t',
    't/exceptions/frame-leak.t',
    't/exceptions/meta-role.t',
    't/exceptions/metaclass.t',
    't/exceptions/moose-exporter.t',
    't/exceptions/moose-meta-attribute-native-traits.t',
    't/exceptions/moose-meta-class-immutable-trait.t',
    't/exceptions/moose-meta-method-accessor-native-array.t',
    't/exceptions/moose-meta-method-accessor-native-collection.t',
    't/exceptions/moose-meta-method-accessor-native-grep.t',
    't/exceptions/moose-meta-method-accessor-native-hash-set.t',
    't/exceptions/moose-meta-method-accessor-native-hash.t',
    't/exceptions/moose-meta-method-accessor-native-string-match.t',
    't/exceptions/moose-meta-method-accessor-native-string-replace.t',
    't/exceptions/moose-meta-method-accessor-native-string-substr.t',
    't/exceptions/moose-meta-method-accessor-native.t',
    't/exceptions/moose-meta-method-accessor.t',
    't/exceptions/moose-meta-method-augmented.t',
    't/exceptions/moose-meta-method-constructor.t',
    't/exceptions/moose-meta-method-delegation.t',
    't/exceptions/moose-meta-method-destructor.t',
    't/exceptions/moose-meta-method-overridden.t',
    't/exceptions/moose-meta-role-application-rolesummation.t',
    't/exceptions/moose-meta-role-application-toclass.t',
    't/exceptions/moose-meta-role-application-torole.t',
    't/exceptions/moose-meta-role-application.t',
    't/exceptions/moose-meta-role-attribute.t',
    't/exceptions/moose-meta-role-composite.t',
    't/exceptions/moose-meta-typecoercion-union.t',
    't/exceptions/moose-meta-typecoercion.t',
    't/exceptions/moose-meta-typeconstraint-enum.t',
    't/exceptions/moose-meta-typeconstraint-parameterizable.t',
    't/exceptions/moose-meta-typeconstraint-parameterized.t',
    't/exceptions/moose-meta-typeconstraint-registry.t',
    't/exceptions/moose-meta-typeconstraint.t',
    't/exceptions/moose-role.t',
    't/exceptions/moose-util-metarole.t',
    't/exceptions/moose-util-typeconstraints.t',
    't/exceptions/moose.t',
    't/exceptions/object.t',
    't/exceptions/overload.t',
    't/exceptions/rt-92818.t',
    't/exceptions/rt-94795.t',
    't/exceptions/stringify.t',
    't/exceptions/traits.t',
    't/exceptions/typeconstraints.t',
    't/exceptions/util.t',
    't/immutable/apply_roles_to_immutable.t',
    't/immutable/buildargs.t',
    't/immutable/constructor_is_not_moose.t',
    't/immutable/constructor_is_wrapped.t',
    't/immutable/default_values.t',
    't/immutable/definition_context.t',
    't/immutable/immutable_constructor_error.t',
    't/immutable/immutable_destroy.t',
    't/immutable/immutable_meta_class.t',
    't/immutable/immutable_metaclass_with_traits.t',
    't/immutable/immutable_moose.t',
    't/immutable/immutable_roundtrip.t',
    't/immutable/immutable_trigger_from_constructor.t',
    't/immutable/inline_close_over.t',
    't/immutable/inline_fallbacks.t',
    't/immutable/inlined_constructors_n_types.t',
    't/immutable/multiple_demolish_inline.t',
    't/lib/Bar.pm',
    't/lib/Bar7/Meta/Trait.pm',
    't/lib/Bar7/Meta/Trait2.pm',
    't/lib/Foo.pm',
    't/lib/Moose/Meta/Attribute/Custom/Bar.pm',
    't/lib/Moose/Meta/Attribute/Custom/Foo.pm',
    't/lib/Moose/Meta/Attribute/Custom/Trait/Bar.pm',
    't/lib/Moose/Meta/Attribute/Custom/Trait/Foo.pm',
    't/lib/MyExporter.pm',
    't/lib/MyMetaclassRole.pm',
    't/lib/MyMooseA.pm',
    't/lib/MyMooseB.pm',
    't/lib/MyMooseObject.pm',
    't/lib/NoInlineAttribute.pm',
    't/lib/Overloading/ClassConsumesRoleConsumesOverloads.pm',
    't/lib/Overloading/ClassWithCombiningRole.pm',
    't/lib/Overloading/ClassWithOneRole.pm',
    't/lib/Overloading/CombiningClass.pm',
    't/lib/Overloading/CombiningRole.pm',
    't/lib/Overloading/RoleConsumesOverloads.pm',
    't/lib/Overloading/RoleWithOverloads.pm',
    't/lib/Overloading/RoleWithoutOverloads.pm',
    't/lib/OverloadingTests.pm',
    't/lib/Real/Package.pm',
    't/lib/Role/BreakOnLoad.pm',
    't/lib/Role/Child.pm',
    't/lib/Role/Interface.pm',
    't/lib/Role/Parent.pm',
    't/metaclasses/create_anon_with_required_attr.t',
    't/metaclasses/custom_attr_meta_as_role.t',
    't/metaclasses/custom_attr_meta_with_roles.t',
    't/metaclasses/easy_init_meta.t',
    't/metaclasses/export_with_prototype.t',
    't/metaclasses/exporter_also_with_trait.t',
    't/metaclasses/exporter_meta_lookup.t',
    't/metaclasses/exporter_sub_names.t',
    't/metaclasses/goto_moose_import.t',
    't/metaclasses/immutable_metaclass_compat_bug.t',
    't/metaclasses/meta_name.t',
    't/metaclasses/metaclass_compat.t',
    't/metaclasses/metaclass_compat_no_fixing_bug.t',
    't/metaclasses/metaclass_compat_role_conflicts.t',
    't/metaclasses/metaclass_parameterized_traits.t',
    't/metaclasses/metaclass_traits.t',
    't/metaclasses/metarole.t',
    't/metaclasses/metarole_combination.t',
    't/metaclasses/metarole_on_anon.t',
    't/metaclasses/metarole_w_metaclass_pm.t',
    't/metaclasses/metaroles_of_metaroles.t',
    't/metaclasses/moose_exporter.t',
    't/metaclasses/moose_exporter_trait_aliases.t',
    't/metaclasses/moose_for_meta.t',
    't/metaclasses/moose_nonmoose_metatrait_init_order.t',
    't/metaclasses/moose_nonmoose_moose_chain_init_meta.t',
    't/metaclasses/moose_w_metaclass.t',
    't/metaclasses/new_metaclass.t',
    't/metaclasses/new_object_BUILD.t',
    't/metaclasses/overloading.t',
    't/metaclasses/reinitialize.t',
    't/metaclasses/use_base_of_moose.t',
    't/moose_util/apply_roles.t',
    't/moose_util/create_alias.t',
    't/moose_util/ensure_all_roles.t',
    't/moose_util/method_mod_args.t',
    't/moose_util/moose_util.t',
    't/moose_util/moose_util_does_role.t',
    't/moose_util/moose_util_search_class_by_role.t',
    't/moose_util/resolve_alias.t',
    't/moose_util/with_traits.t',
    't/native_traits/array_coerce.t',
    't/native_traits/array_from_role.t',
    't/native_traits/array_subtypes.t',
    't/native_traits/array_trigger.t',
    't/native_traits/collection_with_roles.t',
    't/native_traits/custom_instance.t',
    't/native_traits/hash_coerce.t',
    't/native_traits/hash_subtypes.t',
    't/native_traits/hash_trigger.t',
    't/native_traits/remove_attribute.t',
    't/native_traits/shallow_clone.t',
    't/native_traits/trait_array.t',
    't/native_traits/trait_bool.t',
    't/native_traits/trait_code.t',
    't/native_traits/trait_counter.t',
    't/native_traits/trait_hash.t',
    't/native_traits/trait_number.t',
    't/native_traits/trait_string.t',
    't/recipes/basics_bankaccount_methodmodifiersandsubclassing.t',
    't/recipes/basics_binarytree_attributefeatures.t',
    't/recipes/basics_company_subtypes.t',
    't/recipes/basics_datetime_extendingnonmooseparent.t',
    't/recipes/basics_document_augmentandinner.t',
    't/recipes/basics_genome_overloadingsubtypesandcoercion.t',
    't/recipes/basics_http_subtypesandcoercion.t',
    't/recipes/basics_point_attributesandsubclassing.t',
    't/recipes/extending_debugging_baseclassrole.t',
    't/recipes/extending_mooseish_moosesugar.t',
    't/recipes/legacy_debugging_baseclassreplacement.t',
    't/recipes/legacy_labeled_attributemetaclass.t',
    't/recipes/meta_globref_instancemetaclass.t',
    't/recipes/meta_labeled_attributetrait.t',
    't/recipes/meta_privateorpublic_methodmetaclass.t',
    't/recipes/meta_table_metaclasstrait.t',
    't/recipes/roles_applicationtoinstance.t',
    't/recipes/roles_comparable_codereuse.t',
    't/recipes/roles_restartable_advancedcomposition.t',
    't/roles/anonymous_roles.t',
    't/roles/application_toclass.t',
    't/roles/apply_role.t',
    't/roles/build.t',
    't/roles/conflict_many_methods.t',
    't/roles/create_role.t',
    't/roles/create_role_subclass.t',
    't/roles/empty_method_modifiers_meta_bug.t',
    't/roles/extending_role_attrs.t',
    't/roles/free_anonymous_roles.t',
    't/roles/imported_required_method.t',
    't/roles/meta_role.t',
    't/roles/method_aliasing_in_composition.t',
    't/roles/method_exclusion_in_composition.t',
    't/roles/method_modifiers.t',
    't/roles/methods.t',
    't/roles/more_alias_and_exclude.t',
    't/roles/more_role_edge_cases.t',
    't/roles/new_meta_role.t',
    't/roles/overloading_combine_to_class.t',
    't/roles/overloading_combine_to_instance.t',
    't/roles/overloading_combine_to_role.t',
    't/roles/overloading_composition_errors.t',
    't/roles/overloading_remove_attributes_bug.t',
    't/roles/overloading_to_class.t',
    't/roles/overloading_to_instance.t',
    't/roles/overloading_to_role.t',
    't/roles/overriding.t',
    't/roles/reinitialize_anon_role.t',
    't/roles/role.t',
    't/roles/role_attr_application.t',
    't/roles/role_attribute_conflict.t',
    't/roles/role_attrs.t',
    't/roles/role_compose_requires.t',
    't/roles/role_composite.t',
    't/roles/role_composite_exclusion.t',
    't/roles/role_composition_attributes.t',
    't/roles/role_composition_conflict_detection.t',
    't/roles/role_composition_errors.t',
    't/roles/role_composition_method_mods.t',
    't/roles/role_composition_methods.t',
    't/roles/role_composition_override.t',
    't/roles/role_composition_req_methods.t',
    't/roles/role_conflict_detection.t',
    't/roles/role_conflict_edge_cases.t',
    't/roles/role_consumers.t',
    't/roles/role_exclusion.t',
    't/roles/role_exclusion_and_alias_bug.t',
    't/roles/role_for_combination.t',
    't/roles/roles_and_method_cloning.t',
    't/roles/roles_and_req_method_edge_cases.t',
    't/roles/roles_applied_in_create.t',
    't/roles/run_time_role_composition.t',
    't/roles/runtime_roles_and_attrs.t',
    't/roles/runtime_roles_and_nonmoose.t',
    't/roles/runtime_roles_w_params.t',
    't/roles/use_base_does.t',
    't/test_moose/test_moose.t',
    't/test_moose/test_moose_does_ok.t',
    't/test_moose/test_moose_has_attribute_ok.t',
    't/test_moose/test_moose_meta_ok.t',
    't/test_moose/with_immutable.t',
    't/todo_tests/exception_reflects_failed_constraint.t',
    't/todo_tests/immutable_n_around.t',
    't/todo_tests/moose_and_threads.t',
    't/todo_tests/replacing_super_methods.t',
    't/todo_tests/required_role_accessors.t',
    't/todo_tests/role_insertion_order.t',
    't/todo_tests/various_role_features.t',
    't/todo_tests/wrong-inner.t',
    't/type_constraints/advanced_type_creation.t',
    't/type_constraints/class_subtypes.t',
    't/type_constraints/class_type_constraint.t',
    't/type_constraints/coerced_parameterized_types.t',
    't/type_constraints/container_type_coercion.t',
    't/type_constraints/container_type_constraint.t',
    't/type_constraints/custom_parameterized_types.t',
    't/type_constraints/custom_type_errors.t',
    't/type_constraints/define_type_twice_throws.t',
    't/type_constraints/duck_type_handles.t',
    't/type_constraints/duck_types.t',
    't/type_constraints/enum.t',
    't/type_constraints/inlining.t',
    't/type_constraints/match_type_operator.t',
    't/type_constraints/maybe_type_constraint.t',
    't/type_constraints/misc_type_tests.t',
    't/type_constraints/name_conflicts.t',
    't/type_constraints/normalize_type_name.t',
    't/type_constraints/parameterize_from.t',
    't/type_constraints/role_type_constraint.t',
    't/type_constraints/subtype_auto_vivify_parent.t',
    't/type_constraints/subtyping_parameterized_types.t',
    't/type_constraints/subtyping_union_types.t',
    't/type_constraints/throw_error.t',
    't/type_constraints/type_coersion_on_lazy_attributes.t',
    't/type_constraints/type_names.t',
    't/type_constraints/type_notation_parser.t',
    't/type_constraints/types_and_undef.t',
    't/type_constraints/union_is_a_type_of.t',
    't/type_constraints/union_types.t',
    't/type_constraints/union_types_and_coercions.t',
    't/type_constraints/util_find_type_constraint.t',
    't/type_constraints/util_more_type_coercion.t',
    't/type_constraints/util_std_type_constraints.t',
    't/type_constraints/util_type_coercion.t',
    't/type_constraints/util_type_constraints.t',
    't/type_constraints/util_type_constraints_export.t',
    't/type_constraints/util_type_reloading.t',
    't/type_constraints/with-specio.t',
    't/type_constraints/with-type-tiny.t',
    't/zzz-check-breaks.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
